function varargout = PIMbinPSI(varargin)
% PIMbinPSI Matlab GUI for demonstrating use of PIMsoft PSI binary files
% The GUI has the following functionality:
% * Choose which type of image to view in the Image type panel
% * Choose whether to downscale the image by providing a value in the 
%   Window size field
% * Scaling factors are shown in the scaling factors panel. These cannot be 
%   changed, they are automatically loaded from the binary file.
% * Choose which color map to use for the image. The color map is scaled 
%   automatically by default, for manual scaling enter a maximum value in the 
%   field above the color bar
% * Choose which image to show by using the slider bar under the image or by 
%   entering a value in the field under the slider bar
% * Define a region of interest (ROI) by drawing a rectangle with the mouse 
%   on the image. The average values within the ROI will be shown in the 
%   Region of interest panel.
% * Copy the image to an external window using the New figure window button. 
% * From the new figure window, the image can be copied, saved or printed.

% Version 1.0   Perimed 2010

% Last Modified by GUIDE v2.5 09-Dec-2010 09:35:14

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @PIMbinPSI_OpeningFcn, ...
                   'gui_OutputFcn',  @PIMbinPSI_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT


% --- Executes just before PIMbinPSI is made visible.
function PIMbinPSI_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to PIMbinPSI (see VARARGIN)

% Choose default command line output for PIMbinPSI
handles.output = hObject;

% Update handles structure
guidata(hObject, handles);

% UIWAIT makes PIMbinPSI wait for user response (see UIRESUME)
% uiwait(handles.FigureWindow);


% --- Outputs from this function are returned to the command line.
function varargout = PIMbinPSI_OutputFcn(hObject, eventdata, handles) 
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get default command line output from handles structure
varargout{1} = handles.output;


% --- Executes on button press in OpenDataButton.
% Load a data file
function OpenDataButton_Callback(hObject, eventdata, handles)
% hObject    handle to OpenDataButton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


[filename pathname]=uigetfile({'*.dat' 'PIMsoft binary data file'});
data=PIMSoftBinary;     % create a variable of the PIMSoftBinary class
data.OpenFile([pathname filename]); % use the OpenFile method
if isfield(handles,'data')  % if file was opened before
    handles.data.delete();  % close open file
end
% save the data in the handles structure for availability in the whole GUI
handles.data=data;  
if ~strncmp(data.fileType,'PSI',3)  % Check file type
    errordlg(['Error: this script only reads PSI files. The file you are '...
        'trying to read is of type ' data.fileType])
    return
end
if data.fileVersion~=1  %Check file version
    warndlg(['Warning: unknown file version (' num2str(data.fileVersion) ...
        '). May lead to unexpected results!'])
end

% Initialize the GUI
set(handles.CoherenceFactorValue,'String',num2str(data.coherenceFactor))
set(handles.SignalGainValue,'String',num2str(data.signalGain))
windowSize=str2double(get(handles.WindowSizeValue,'String'));
ShowImage(handles,'Intensity',windowSize,1)
images=data.numberOfImages;
set(handles.FrameSelectSlider,'Value',1,'Min',1,'Max',images,...
    'SliderStep',[1/images 10/images])
set(handles.FrameSelectValue,'String','1')
set(handles.MaxFrameText,'String',num2str(images))
set(findobj(handles.FigureWindow,'Type','uicontrol'),'Enable','on')
set(handles.CoherenceFactorValue,'Enable','off')
set(handles.SignalGainValue,'Enable','off')
guidata(hObject, handles);

% --- Executes on slider movement.
% Changes the frame shown according to the slider position
function FrameSelectSlider_Callback(hObject, eventdata, handles)
% hObject    handle to FrameSelectSlider (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'Value') returns position of slider
%        get(hObject,'Min') and get(hObject,'Max') to determine range of slider

frame=round(get(hObject, 'Value'));
set(handles.FrameSelectValue,'String',num2str(frame))
imType=get(get(handles.ImageTypePanel,'SelectedObject'),'String');
windowSize=str2double(get(handles.WindowSizeValue,'String'));
ShowImage(handles,imType,windowSize,frame)

% If a ROI exists, it needs to be recalculated
ROI=findobj(handles.ImageAxes,'Type','line');
if ~isempty(ROI)
    x=get(ROI,'XData');
    y=get(ROI,'YData');
    CalculateROI(handles,x,y)
end




% --- Executes during object creation, after setting all properties.
function FrameSelectSlider_CreateFcn(hObject, eventdata, handles)
% hObject    handle to FrameSelectSlider (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: slider controls usually have a light gray background.
if isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor',[.9 .9 .9]);
end



function CoherenceFactorValue_Callback(hObject, eventdata, handles)
% hObject    handle to CoherenceFactorValue (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of CoherenceFactorValue as text
%        str2double(get(hObject,'String')) returns contents of CoherenceFactorValue as a double


% --- Executes during object creation, after setting all properties.
function CoherenceFactorValue_CreateFcn(hObject, eventdata, handles)
% hObject    handle to CoherenceFactorValue (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function SignalGainValue_Callback(hObject, eventdata, handles)
% hObject    handle to SignalGainValue (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of SignalGainValue as text
%        str2double(get(hObject,'String')) returns contents of SignalGainValue as a double


% --- Executes during object creation, after setting all properties.
function SignalGainValue_CreateFcn(hObject, eventdata, handles)
% hObject    handle to SignalGainValue (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% Shows the chosen image on the screen
function ShowImage(handles,type,windowSize,num)
data=handles.data;

% If a ROI exists, save it for later redrawing
ROI=findobj(handles.ImageAxes,'Type','line');
if ~isempty(ROI)
    x=get(ROI,'XData');
    y=get(ROI,'YData');
end

% Show the correct image type
switch type
    case 'Intensity'
        ImData=data.getDCFrame(num);
        if windowSize>1
            ImData=blockAverageDownscale(ImData,windowSize);
        end
        hImage=imagesc(ImData);
    case 'Variance'
        ImData=data.getVarianceFrame(num);
        if windowSize>1
            ImData=blockAverageDownscale(ImData,windowSize);
        end
        hImage=imagesc(ImData);
    case 'Contrast'
        ImVariance=data.getVarianceFrame(num);
        ImIntensity=data.getDCFrame(num);
        CoherenceFactor=data.coherenceFactor;
        ImData=calcContrast(ImVariance, ImIntensity, CoherenceFactor,windowSize);
        hImage=imagesc(real(ImData));
    case 'Perfusion'
        ImVariance=data.getVarianceFrame(num);
        ImIntensity=data.getDCFrame(num);
        CoherenceFactor=data.coherenceFactor;
        SignalGain=data.signalGain;
        ImData=calcPerfusion(ImVariance, ImIntensity, CoherenceFactor, SignalGain, windowSize);
        hImage=imagesc(ImData);
end
colorbar    % show the colorbar
axis equal  % prevent stretching of the image in one direction

% If a ROI existed, redraw it
if ~isempty(ROI)
    hold on
    axis manual
    plot(x,y,'r')
    hold off
end

% Define the function for drawing a ROI
set(hImage,'ButtonDownFcn',{@ImageCallback,handles})

% Set the color scale
maxColor=str2double(get(handles.MaxColorValue,'String'));
if ~isnan(maxColor)
    caxis([0 maxColor])
else
    caxis auto
end



% --- Executes when selected object is changed in ImageTypePanel.
% Show the new image when another type is chosen
function ImageTypePanel_SelectionChangeFcn(hObject, eventdata, handles)
% hObject    handle to the selected object in ImageTypePanel 
% eventdata  structure with the following fields (see UIBUTTONGROUP)
%	EventName: string 'SelectionChanged' (read only)
%	OldValue: handle of the previously selected object or empty if none was selected
%	NewValue: handle of the currently selected object
% handles    structure with handles and user data (see GUIDATA)

ImageTypeSelection=get(eventdata.NewValue,'String');
windowSize=str2double(get(handles.WindowSizeValue,'String'));
frame=str2double(get(handles.FrameSelectValue, 'String'));
ShowImage(handles,ImageTypeSelection,windowSize,frame)


% Show the new image when a new frame number is chosen
function FrameSelectValue_Callback(hObject, eventdata, handles)
% hObject    handle to FrameSelectValue (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of FrameSelectValue as text
%        str2double(get(hObject,'String')) returns contents of FrameSelectValue as a double

frame=str2double(get(hObject, 'String'));
set(handles.FrameSelectSlider,'Value',frame)
imType=get(get(handles.ImageTypePanel,'SelectedObject'),'String');
windowSize=str2double(get(handles.WindowSizeValue,'String'));
ShowImage(handles,imType,windowSize,frame)


% --- Executes during object creation, after setting all properties.
function FrameSelectValue_CreateFcn(hObject, eventdata, handles)
% hObject    handle to FrameSelectValue (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% Show the new image if the window size is changed
function WindowSizeValue_Callback(hObject, eventdata, handles)
% hObject    handle to WindowSizeValue (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of WindowSizeValue as text
%        str2double(get(hObject,'String')) returns contents of WindowSizeValue as a double

frame=str2double(get(handles.FrameSelectValue, 'String'));
set(handles.FrameSelectSlider,'Value',frame)
imType=get(get(handles.ImageTypePanel,'SelectedObject'),'String');
windowSize=str2double(get(hObject,'String'));

% If a ROI exists, it is removed
ROI=findobj(handles.ImageAxes,'Type','line');
if ~isempty(ROI)
    delete(ROI)
    set(handles.ROIIntensityValue,'String','')
    set(handles.ROIVarianceValue,'String','')
    set(handles.ROIContrastValue,'String','')
    set(handles.ROIPerfusionValue,'String','')
end
ShowImage(handles,imType,windowSize,frame)


% --- Executes during object creation, after setting all properties.
function WindowSizeValue_CreateFcn(hObject, eventdata, handles)
% hObject    handle to WindowSizeValue (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on selection change in ColormapMenu.
function ColormapMenu_Callback(hObject, eventdata, handles)
% hObject    handle to ColormapMenu (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = cellstr(get(hObject,'String')) returns ColormapMenu contents as cell array
%        contents{get(hObject,'Value')} returns selected item from ColormapMenu

maps=get(hObject,'String');
map=cell2mat(maps(get(hObject,'Value')));
colormap(map)

% --- Executes during object creation, after setting all properties.
function ColormapMenu_CreateFcn(hObject, eventdata, handles)
% hObject    handle to ColormapMenu (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: popupmenu controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% Update the image if the color scaling is changed
function MaxColorValue_Callback(hObject, eventdata, handles)
% hObject    handle to MaxColorValue (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of MaxColorValue as text
%        str2double(get(hObject,'String')) returns contents of MaxColorValue as a double

frame=str2double(get(handles.FrameSelectValue, 'String'));
set(handles.FrameSelectSlider,'Value',frame)
imType=get(get(handles.ImageTypePanel,'SelectedObject'),'String');
windowSize=str2double(get(handles.WindowSizeValue,'String'));
ShowImage(handles,imType,windowSize,frame)

% --- Executes during object creation, after setting all properties.
function MaxColorValue_CreateFcn(hObject, eventdata, handles)
% hObject    handle to MaxColorValue (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% Draw an ROI when user drags mouse on image
function ImageCallback(hObject, eventdata, handles)
% If previous ROI exists, remove it
previousROI=findobj(handles.ImageAxes,'Type','line');
if ~isempty(previousROI)
    delete(previousROI)
end

% Get ROI coordinates
point1=get(handles.ImageAxes,'CurrentPoint');
rbbox                               % draw the rectangle
point2=get(handles.ImageAxes,'CurrentPoint');
point1 = point1(1,1:2);             % extract x and y
point2 = point2(1,1:2);
p1 = min(point1,point2);            % calculate locations
offset = abs(point1-point2);        % and dimensions
x = [p1(1) p1(1)+offset(1) p1(1)+offset(1) p1(1) p1(1)]';
y = [p1(2) p1(2) p1(2)+offset(2) p1(2)+offset(2) p1(2)]';
hold on
axis manual
plot(x,y,'r');                      % redraw in dataspace units
hold off
CalculateROI(handles,x,y)

% Calculate the ROI values
function CalculateROI(handles,x,y)
if ((x(1)-x(2))*(y(1)-y(3)))~=0 % Only if ROI has final area
    windowSize=str2double(get(handles.WindowSizeValue,'String'));
    frame=str2double(get(handles.FrameSelectValue, 'String'));
    data=handles.data;
    CoherenceFactor=data.coherenceFactor;
    SignalGain=data.signalGain;
    Intensity=data.getDCFrame(frame);
    Variance=data.getVarianceFrame(frame);
    if windowSize>1
        Intensity=blockAverageDownscale(Intensity,windowSize);
        Variance=blockAverageDownscale(Variance,windowSize);
    end
    X=repmat(1:size(Intensity,2),size(Intensity,1),1);
    Y=repmat((1:size(Intensity,1))',1,size(Intensity,2));
    Mask=inpolygon(X,Y,x,y); % create mask
    Intensity=mean(mean(Intensity(Mask)));
    Variance=mean(mean(Variance(Mask)));
    Contrast=calcContrast(Variance,Intensity,CoherenceFactor);
    Perfusion=calcPerfusion(Variance,Intensity,CoherenceFactor,SignalGain);
    set(handles.ROIIntensityValue,'String',num2str(Intensity,'%12.5g'))
    set(handles.ROIVarianceValue,'String',num2str(Variance,'%12.5g'))
    set(handles.ROIContrastValue,'String',num2str(Contrast,'%12.3g'))
    set(handles.ROIPerfusionValue,'String',num2str(Perfusion,'%12.2f'))
    set(findobj(handles.ROIPanel,'Style','Text','ForegroundColor',[0 0 0]),...
        'Visible','on') % Show ROI values
    set(findobj(handles.ROIPanel,'Style','Text','ForegroundColor',[1 0 0]),...
        'Visible','off')% Hide help text
else % ROI with no area equals no ROI
    set(handles.ROIIntensityValue,'String','')
    set(handles.ROIVarianceValue,'String','')
    set(handles.ROIContrastValue,'String','')
    set(handles.ROIPerfusionValue,'String','')
    set(findobj(handles.ROIPanel,'Style','Text','ForegroundColor',[0 0 0]),...
        'Visible','off')% Hide ROI values
    set(findobj(handles.ROIPanel,'Style','Text','ForegroundColor',[1 0 0]),...
        'Visible','on') % Show help text
end


% --- Executes on button press in NewFigButton.
% Opens a new figure window and redraws the image
function NewFigButton_Callback(hObject, eventdata, handles)
% hObject    handle to NewFigButton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

hFig=figure;
imageData=get(findobj(handles.ImageAxes,'Type','image'),'CData');
imagesc(imageData)
colorbar
axis equal
ROI=findobj(handles.ImageAxes,'Type','line');
if ~isempty(ROI)
    x=get(ROI,'XData');
    y=get(ROI,'YData');
    hold on
    axis manual
    plot(x,y,'r')
    hold off
end
maxColor=str2double(get(handles.MaxColorValue,'String'));
if ~isnan(maxColor)
    caxis([0 maxColor])
else
    caxis auto
end
