function [ outImage ] = blockAverageDownscale( inImage, n )
% [ outImage ] = blockAveargeDownscale( inImage, n )
% Downscales an image (variance or intensity) by averaging over blocks with 
% size n*n. Note: do not use this to directly average contrast or perfusion
% images. Instead average the variance and intensity images and recalculate 
% the contrast and/or perfusion images from those.

% Version 1.0   Perimed 2010

outHeight = floor(size(inImage,1)/n);           % Number of blocks in height
Heightstart=floor(rem(size(inImage,1),n)/2)-1;  % Center blocks on image
Heightstart=max(Heightstart,1);                 % Start at minimum 1
outWidth = floor(size(inImage,2)/n);            % Number of blocks in width
Widthstart=floor(rem(size(inImage,2),n)/2)-1;   % Center blocks on image
Widthstart=max(Widthstart,1);                   % Start at minimum 1
outImage = zeros ( outHeight, outWidth );       % Pre-allocate image
for i = 1:outHeight;
    for j = 1:outWidth;
        imageBlock = inImage(Heightstart+(i-1)*n+1:(i)*n, Widthstart+(j-1)*n+1:j*n);
		imageBlock = imageBlock(~isnan(imageBlock));
        outImage(i,j) = mean( imageBlock(:) );
    end
end