function [ ContrastImage ] = calcContrast( VarianceImage, IntensityImage, CoherenceFactor, WindowSize )
%[ ContrastImage ] = calcContrast( VarianceImage, IntensityImage, CoherenceFactor, window )
% This function calculates the contrast from a variance and an intensity
% image. The coherence factor is needed to scale the contrast, if it is not
% given, a default value of 2.5 will be used. If WindowSize is given, the
% contrast will be scaled by averaging over blocks of n*n pixels, where
% n=windowSize. This is done by first downscaling the variance and
% intensity images and then calculating the contrast, because scaling down
% the contrast image directly can give incorrect values.

% Version 1.0   Perimed 2010

if nargin<2
    errordlg('Error: contrast calculation needs variance and intensity image.')
    ContrastImage=[];
    return
elseif nargin==2
    warndlg('Warning: no coherence factor specified. Assuming 2.5')
    CoherenceFactor=2.5;
end

if nargin==3||WindowSize==1 % Calculation of single values
    VarSign=sign(VarianceImage);
    ContrastImage = sqrt(abs(VarianceImage))./IntensityImage*CoherenceFactor;
    ContrastImage = VarSign.*ContrastImage;
else                        % Downscale image
    AveragedVarianceImage = blockAverageDownscale(VarianceImage, WindowSize);
    VarSign=sign(AveragedVarianceImage);
    AveragedIntensityImage = blockAverageDownscale(IntensityImage, WindowSize);
    ContrastImage = sqrt(abs(AveragedVarianceImage))./AveragedIntensityImage*CoherenceFactor;
    ContrastImage = VarSign.*ContrastImage;
end

end

